"""Top-level package for clustergrammer2_core."""

from .clustergrammer_fun import *

net = Network()

__author__ = """Nicolas Fernandez"""
__email__ = 'nickfloresfernandez@gmail.com'
__version__ = '0.1.0'
